from erdpy.dependencies.install import install_group, install_module, get_install_directory, get_module_by_key, is_installed

__all__ = ["install_group", "install_module", "get_install_directory", "get_module_by_key", "is_installed"]
